#include "cardappz.h"
#include <assert.h>
#include <stdio.h>

unsigned char CORPID_MASTER_KEY0[16] =
  { 0x00, 0x00, 0x0B, 0x00, 0x00, 0x50, 0x41, 0x01, 0x00, 0x00, 0x1B, 0x00,
  0x00, 0x50, 0x41, 0x01
};
unsigned char CORPID_MASTER_KEY1[16] =
  { 0x00, 0x00, 0x0A, 0x00, 0x00, 0x50, 0x41, 0x01, 0x00, 0x00, 0x1A, 0x00,
  0x00, 0x50, 0x41, 0x01
};
unsigned char CORPID_SIGNATURE_KEY[16] =
  { 0x00, 0x00, 0x0F, 0x00, 0x00, 0x50, 0x41, 0x01, 0x00, 0x00, 0x1F, 0x00,
  0x00, 0x50, 0x41, 0x01
};


int DisplayCorpid(CARDAPPZ_CTX_ST * ctx)
{
  CORPID_IDENTITY_ST identity;
  unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ];
  unsigned char picture[CORPID_PICTURE_MAX_SZ];
  unsigned char accesscontrol[48];
  CORPID_OPTIONSSIGN_ST optionssing;

  int rc;

  if (ctx == NULL)
    return CARDAPPZ_INVALID_CONTEXT;

  /* Select the corpid application */
  printf("Entering CorpID application\n");
  rc = SelectCorpid(ctx);
  if (rc)
    goto done;

  rc = ReadCorpidIdentity(ctx, &identity);
  if (rc)
    goto done;

  rc = ReadCorpidFingerprints(ctx, fingerprints);
  if (rc)
    goto done;

  rc = ReadCorpidPicture(ctx, picture);
  if (rc)
    goto done;

  rc = ReadCorpidAccessControl(ctx, accesscontrol);
  if (rc)
    goto done;

  rc = ReadCorpidOptionsSign(ctx, &optionssing);
  if (rc)
    goto done;


  rc = CARDAPPZ_OK;

done:
  if (rc)
    printf("Error %d in DisplayCorpid\n", rc);
  return rc;


}

int SelectCorpid(CARDAPPZ_CTX_ST * ctx)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireSelectCorpid(ctx);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadCorpidIdentity(CARDAPPZ_CTX_ST * ctx, CORPID_IDENTITY_ST * identity)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadCorpidIdentity(ctx, identity);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadCorpidFingerprints(CARDAPPZ_CTX_ST * ctx,
                           unsigned char
                           fingerprints[CORPID_FINGERPRINTS_MAX_SZ])
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadCorpidFingerprints(ctx, fingerprints);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadCorpidPicture(CARDAPPZ_CTX_ST * ctx,
                      unsigned char picture[CORPID_PICTURE_MAX_SZ])
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadCorpidPicture(ctx, picture);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadCorpidAccessControl(CARDAPPZ_CTX_ST * ctx,
                            unsigned char accesscontrol[48])
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadCorpidAccessControl(ctx, accesscontrol);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadCorpidOptionsSign(CARDAPPZ_CTX_ST * ctx,
                          CORPID_OPTIONSSIGN_ST * optionssign)
{
  int rc;

  assert(ctx != NULL);

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc = DesfireReadCorpidOptionsSign(ctx, optionssign);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Convert to computer endianess */
  if (optionssign != NULL)
    optionssign->PictureSize = CtoHs(optionssign->PictureSize);

  return rc;
}

int WriteCorpid(CARDAPPZ_CTX_ST * ctx, CORPID_IDENTITY_ST * identity,
                unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ],
                unsigned char picture[CORPID_PICTURE_MAX_SZ],
                unsigned char accesscontrol[48],
                CORPID_OPTIONSSIGN_ST * optionssign)
{
  int rc;

  assert(ctx != NULL);

  /* Ensure endianess on card */
  if (optionssign != NULL)
    optionssign->PictureSize = HtoCs(optionssign->PictureSize);

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc =
        DesfireWriteCorpid(ctx, identity, fingerprints, picture,
                           accesscontrol, optionssign);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Back to computer endianess */
  if (optionssign != NULL)
    optionssign->PictureSize = CtoHs(optionssign->PictureSize);

  return rc;
}
