#include "cardappz.h"
#include "hardware/springprox/springprox.h"
#include <stdio.h>
#include <conio.h>
#include <assert.h>

int MainMenu(CARDAPPZ_CTX_ST * ctx);
int SelectCard(CARDAPPZ_CTX_ST * ctx);
int SelectDesfireSAM(CARDAPPZ_CTX_ST * ctx);
int CloseCard(CARDAPPZ_CTX_ST * ctx);

int main(int argc, char **argv)
{
  int rc;

  CARDAPPZ_CTX_ST CTX = { 0 };
  CARDAPPZ_CTX_ST *ctx = &CTX;

  printf("SpringCard SDK for K531/SpringProx-CF/CSB Legacy\n");
  printf("\n");
  printf("CARDAPPZ reference application\n");
  printf("------------------------------n");
  printf("www.springcard.com\n\n");
  printf("NOTE: This application is only a skeleton, showing only the\n");
  printf("      principles to be understood before designing and implementing\n");
  printf("      a system-wide scheme involving contactless smartcards or memory\n");
  printf("      cards.\n");
  printf("      All explanations are to be found in cardappz.html document.\n\n");

  ctx->trace = DUMPMORE;

  printf("Connection to the reader...\n");
  rc = SPROX_ReaderOpen(NULL);
  if (rc != MI_OK)
  {
    printf("Failed !\n");
    return EXIT_FAILURE;
  }

  /* We try to discover a Desfire SAM in the reader */
  printf("Looking for a SAM card\n");
  rc = SelectDesfireSAM(ctx);
  //if (rc != CARDAPPZ_OK) goto failed;

  /* We wait for a contactless card on the reader */
  printf("Waiting for a contactless card\n");
  rc = SelectCard(ctx);
  if (rc != CARDAPPZ_OK) goto failed;

  /* And now we can enter the interactive mode */
  if (rc == CARDAPPZ_OK)
    MainMenu(ctx);

  return EXIT_SUCCESS;

failed:
  printf("Failed !\n");
  SPROX_ReaderClose();
  return EXIT_FAILURE;
}


int MainMenu(CARDAPPZ_CTX_ST * ctx)
{
  int rc;
  int c;

  assert(ctx != NULL);

  for (;;)
  {
    printf("\n");
    printf("***** MAIN MENU *****\n");
    printf("\n");
    printf("Card management :\n");
    printf("B -- (B)lank the card\n");
    printf("F -- (F)ormat the card\n");
    printf("\n");
    printf("Card applications :\n");
    printf("I -- Corporate (I)D card\n");
    printf("P -- Electronic (P)urse\n");
    printf("T -- Public (T)ransportation\n");
    printf("\n");
    printf("C -- Select another (C)ard\n");
    if (ctx->desfire_sam.found)
    {
      printf("S -- Manage (S)AM\n");
    }
    printf("\n");
    printf("X -- E(X)it this program\n");

    c = toupper(getch());
    switch (c)
    {
      case 'B':
        if (confirm("Erase the whole card", 0))
        {
          printf("*** Erasing the card ***\n");
          rc = Blank(ctx);
          if (rc)
            printf("Failed !\n");
          else
            printf("Success !\n");
          pause();
        }
        break;
      case 'F':
        if (confirm("Create the card structure", 1))
        {
          printf("*** Formatting the card ***\n");
          rc = Format(ctx);
          if (rc)
            printf("Failed !\n");
          else
            printf("Success !\n");
          pause();
        }
        break;

/*
      case 'I' : rc = CorpidMenu(ctx);
                 break;
      case 'P' : rc = PurseMenu(ctx);
                 break;
*/
      case 'T':
        rc = TransportMenu(ctx);
        break;

      case 'S':
        if (ctx->desfire_sam.found)
          rc = DesfireSamMenu(ctx);
        break;

      case 'C':
        CloseCard(ctx);
        rc = SelectCard(ctx);
        if (rc)
          return rc;
        break;

      case 'X':
        CloseCard(ctx);
        return CARDAPPZ_OK;
    }
  }

  return CARDAPPZ_OK;
}

int SelectCard(CARDAPPZ_CTX_ST * ctx)
{
  int i, rc, c;

  assert(ctx != NULL);

  printf("\nWaiting for a card...\nESC to exit\n");

  for (;;)
  {
    if (kbhit())
    {
      c = getch();
      if (c == 27)
        break;
    }

    SPROX_SetConfig(CFG_MODE_ISO_14443_A);
    ctx->card_ident.a.snrlen = sizeof(ctx->card_ident.a.snr);
    rc =
      SPROX_TclA_ActivateAny(ctx->card_ident.a.atq, ctx->card_ident.a.snr,
                             &ctx->card_ident.a.snrlen,
                             ctx->card_ident.a.sak);
    if (rc == CARDAPPZ_OK)
    {
      printf("ATQa=%02X%02X SNR=", ctx->card_ident.a.atq[1],
             ctx->card_ident.a.atq[0]);
      for (i = 0; i < ctx->card_ident.a.snrlen; i++)
        printf("%02X", ctx->card_ident.a.snr[i]);
      printf(" SAK=%02X\n", ctx->card_ident.a.sak);

      ctx->card_ident.a.atslen = sizeof(ctx->card_ident.a.ats);
      rc =
        SPROX_TclA_GetAts(1, ctx->card_ident.a.ats,
                          &ctx->card_ident.a.atslen);

      if (rc == CARDAPPZ_OK)
      {
        printf("ATS=");
        for (i = 0; i < ctx->card_ident.a.atslen; i++)
          printf("%02X", ctx->card_ident.a.ats[i]);
        printf("\n");

        ctx->card_type = CARD_DESFIRE;
        return CARDAPPZ_OK;
      }
    }

    SPROX_SetConfig(CFG_MODE_ISO_14443_B);
    rc = SPROX_TclB_ActivateAny(0, ctx->card_ident.b.atq);
    if (rc == CARDAPPZ_OK)
    {
      printf("ATQb=");
      for (i = 0; i < 11; i++)
        printf("%02X", ctx->card_ident.b.atq[i]);
      printf("\n");

      rc = SPROX_TclB_Attrib(ctx->card_ident.b.atq, 1);
      if (rc == CARDAPPZ_OK)
      {
        ctx->card_type = CARD_JAYCOS;
        return CARDAPPZ_OK;
      }
    }
  }

  printf("\nCancelled...\n");
  return CARDAPPZ_CANCEL;
}

int CloseCard(CARDAPPZ_CTX_ST * ctx)
{
  if (ctx->card_type & CARD_TYPE_14443_4)
  {
    if (ctx->card_type & CARD_TYPE_14443_A)
    {
      SPROX_TclA_Deselect(1);
    } else if (ctx->card_type & CARD_TYPE_14443_B)
    {
      SPROX_TclB_Deselect(1);
    }
  } else if (ctx->card_type & CARD_TYPE_14443_A)
  {
    SPROX_MifStHalt();
  }

  return CARDAPPZ_OK;
}

static unsigned char DESFIRE_SAM_ATR[] =
  { 0x3B, 0xDE, 0x18, 0xFF, 0x81, 0xF1, 0xFB, 0x34, 0x00, 0x1F, 0x07, 0x44,
  0x45, 0x53, 0x46, 0x69, 0x72, 0x65, 0x53, 0x41, 0x4D, 0x56, 0x31, 0x2E,
  0x30, 0xD2
};

int SelectDesfireSAM(CARDAPPZ_CTX_ST * ctx)
{
  unsigned char s;
  int i, rc;
  unsigned short atrlen;
  unsigned char atr[32];

  assert(ctx != NULL);

  for (s = 0; s < 16; s++)
  {
    atrlen = sizeof(atr);
    rc = SPROX_Card_PowerUp_Auto(s, atr, &atrlen);
    if (!rc)
    {
      /* No error = a smartcard has been found */
      printf("Found a smartcard in slot %d\nATR=", s);
      for (i = 0; i < atrlen; i++)
        printf("%02X", atr[i]);
      printf("\n");
      if (atrlen == sizeof(DESFIRE_SAM_ATR))
        if (!memcmp(atr, DESFIRE_SAM_ATR, sizeof(DESFIRE_SAM_ATR)))
        {
          /* This is the Desfire SAM */
          printf("This is the Desfire SAM !\n");
          ctx->desfire_sam.found = 1;
          ctx->desfire_sam.slot = s;
          ctx->desfire_sam.atrlen = (unsigned char) atrlen;
          memcpy(ctx->desfire_sam.atr, atr, atrlen);
          return CARDAPPZ_OK;
        }
    }
  }

  printf("Desfire SAM not found.\n");
  return CARDAPPZ_CANCEL;
}
