#ifndef _CARDAPPZ_TRANSPORT_H_
#define _CARDAPPZ_TRANSPORT_H_

#include "cardappz_types.h"
#include "cardappz_cards.h"

extern unsigned char TRANSPORT_MASTER_KEY0[16];
extern unsigned char TRANSPORT_MASTER_KEY1[16];
extern unsigned char TRANSPORT_SIGNATURE_KEY[16];

#define TRANSPORT_MASTER_KEY0_IDX    7
#define TRANSPORT_MASTER_KEY0_VER    0x23
#define TRANSPORT_MASTER_KEY1_IDX    8
#define TRANSPORT_MASTER_KEY1_VER    0x03
#define TRANSPORT_SIGNATURE_KEY_IDX  9
#define TRANSPORT_SIGNATURE_KEY_VER  0x23

typedef struct
{
  unsigned char Priority;
  unsigned char Network[3];
  unsigned char ProviderID;
  unsigned char CustomerID[4];
  unsigned char Passengers;
  unsigned char Mode;
  unsigned char Areas;
  unsigned char Class;
  unsigned char Counter;
  unsigned char ValidWeekDays;
  unsigned char ValidTotalDays;
  unsigned char StartDate_Year;
  unsigned char StartDate_Month;
  unsigned char StartDate_Day;
  unsigned char StartDate_Hour;
  unsigned char EndDate_Year;
  unsigned char EndDate_Month;
  unsigned char EndDate_Day;
  unsigned char EndDate_Hour;
  unsigned char Certificate[8];
} TRANSPORT_CONTRACT_ST;

typedef struct
{
  unsigned char TimeStamp_Year;
  unsigned char TimeStamp_Month;
  unsigned char TimeStamp_Day;
  unsigned char TimeStamp_Hour;
  unsigned char TimeStamp_Min;
  unsigned char Network[3];
  unsigned char Contract;
  unsigned char Action;
  unsigned char Location[3];
  unsigned char RemainingDays;
  unsigned char Counter;
  unsigned char Charged;
} TRANSPORT_EVENT_ST;

/*
 * Access to application on card
 * -----------------------------
 */

/* cardappz_transport_common.c */
int SelectTransport(CARDAPPZ_CTX_ST *ctx);
int ReadTransportContract(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_CONTRACT_ST *Contract);
int ReadTransportCounter(CARDAPPZ_CTX_ST *ctx, int Index, unsigned long *Value);
int ReadTransportEvent(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_EVENT_ST *Event);
int AppendTransportEvent(CARDAPPZ_CTX_ST *ctx, TRANSPORT_EVENT_ST *Event);
int AppendTransportContract(CARDAPPZ_CTX_ST *ctx, TRANSPORT_CONTRACT_ST *Contract, unsigned long CounterValue);
int EraseTransportContracts(CARDAPPZ_CTX_ST *ctx);

/*
 * Security scheme
 * ---------------
 */

 
/* cardappz_transport_signature.c */
int TransportComputeContractSignature(CARDAPPZ_CTX_ST *ctx, unsigned char Result[8], TRANSPORT_CONTRACT_ST *Contract, const unsigned char KeyValue[16]);
int TransportBuildContractSignature(CARDAPPZ_CTX_ST *ctx, TRANSPORT_CONTRACT_ST *Contract, const unsigned char KeyValue[16]);
int TransportVerifyContractSignature(CARDAPPZ_CTX_ST *ctx, TRANSPORT_CONTRACT_ST *Contract, const unsigned char KeyValue[16]);

/*
 * Human interface
 * ---------------
 */

/* cardappz_transport_menu.c */
int TransportMenu(CARDAPPZ_CTX_ST *ctx);

/* cardappz_transport_usage.c */
int TransportControl(CARDAPPZ_CTX_ST *ctx);
int TransportSubway(CARDAPPZ_CTX_ST *ctx);
int TransportTrain(CARDAPPZ_CTX_ST *ctx);
int TransportTaxiIn(CARDAPPZ_CTX_ST *ctx);
int TransportTaxiOut(CARDAPPZ_CTX_ST *ctx);


#endif
