#include "cardappz.h"
#include <assert.h>
#include <stdio.h>

unsigned char TRANSPORT_MASTER_KEY0[16] =
  { 0x00, 0x00, 0x0B, 0x00, 0x00, 0x50, 0x41, 0x03, 0x00, 0x00, 0x1B, 0x00,
  0x00, 0x50, 0x41, 0x03
};
unsigned char TRANSPORT_MASTER_KEY1[16] =
  { 0x00, 0x00, 0x0A, 0x00, 0x00, 0x50, 0x41, 0x03, 0x00, 0x00, 0x1A, 0x00,
  0x00, 0x50, 0x41, 0x03
};
unsigned char TRANSPORT_SIGNATURE_KEY[16] =
  { 0x00, 0x00, 0x0F, 0x00, 0x00, 0x50, 0x41, 0x03, 0x00, 0x00, 0x1F, 0x00,
  0x00, 0x50, 0x41, 0x03
};

int SelectTransport(CARDAPPZ_CTX_ST * ctx)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireSelectTransport(ctx);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadTransportContract(CARDAPPZ_CTX_ST * ctx, int Index,
                          TRANSPORT_CONTRACT_ST * Contract)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadTransportContract(ctx, Index, Contract);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;

}

int ReadTransportCounter(CARDAPPZ_CTX_ST * ctx, int Index,
                         unsigned long *Value)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadTransportCounter(ctx, Index, Value);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadTransportEvent(CARDAPPZ_CTX_ST * ctx, int Index,
                       TRANSPORT_EVENT_ST * Event)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadTransportEvent(ctx, Index, Event);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int AppendTransportEvent(CARDAPPZ_CTX_ST * ctx, TRANSPORT_EVENT_ST * Event)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireAppendTransportEvent(ctx, Event);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int AppendTransportContract(CARDAPPZ_CTX_ST * ctx,
                            TRANSPORT_CONTRACT_ST * Contract,
                            unsigned long CounterValue)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireAppendTransportContract(ctx, Contract, CounterValue);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int EraseTransportContracts(CARDAPPZ_CTX_ST * ctx)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireEraseTransportContracts(ctx);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int TransportBuildContractSignature(CARDAPPZ_CTX_ST * ctx,
                                    TRANSPORT_CONTRACT_ST * Contract,
                                    const unsigned char KeyValue[16])
{
  assert(Contract != NULL);

  return ComputeSignature(ctx, Contract->Certificate,
                          (unsigned char *) Contract,
                          sizeof(TRANSPORT_CONTRACT_ST) - 8, KeyValue);
}

int TransportVerifyContractSignature(CARDAPPZ_CTX_ST * ctx,
                                     TRANSPORT_CONTRACT_ST * Contract,
                                     const unsigned char KeyValue[16])
{
  unsigned char result[8];
  int rc;

  assert(Contract != NULL);

  rc =
    ComputeSignature(ctx, result, (unsigned char *) Contract,
                     sizeof(TRANSPORT_CONTRACT_ST) - 8, KeyValue);
  if (rc)
    return rc;

  if (memcmp(result, Contract->Certificate, 8))
    return CARDAPPZ_CARD_SIGNATURE_ERROR;

  return CARDAPPZ_OK;
}
