#include <windows.h>
#include "springprox.h"
#include "jaycos.h"

static short JaycosTransmitEx(unsigned char cmd[], unsigned short cmdlen,
                              unsigned char rsp[], unsigned short *rsplen)
{
  SWORD rc;

  /*
     {
     char t[1024];
     unsigned short i;
     for (i=0; i<cmdlen; i++)
     sprintf(&t[2*i], "%02X", cmd[i]);
     MessageBox(0, t, "Oups", 0);
     }
   */

  /* T=CL transmit using SpringProx reader  */
  /* not difficult to change for PC/SC mode */
  if (rsplen != NULL)
  {
    WORD t = (WORD) * rsplen;

    rc = SPROX_Tcl_Exchange(0xFF, cmd, (WORD) cmdlen, rsp, &t);
    *rsplen = t;
  } else
  {
    rc = SPROX_Tcl_Exchange(0xFF, cmd, (WORD) cmdlen, rsp, NULL);
  }

  return rc;
}

static short JaycosTransmit(unsigned char CLA, unsigned char INS,
                            unsigned char P1, unsigned char P2,
                            unsigned char data[], unsigned short LC,
                            signed short LE, unsigned char rsp[],
                            unsigned short *rsplen, unsigned short *status)
{
  unsigned char io_buffer[300];
  unsigned short o_len, i_len;
  unsigned char sw[2];
  int rc;

  if ((LC + 6) >= sizeof(io_buffer))
    return TCL_OTHER_ERR;

  /* Build the ISO 7816-4 APDU */
  io_buffer[0] = CLA;
  io_buffer[1] = INS;
  io_buffer[2] = P1;
  io_buffer[3] = P2;

  o_len = 4;
  if (data != NULL)
  {
    io_buffer[4] = LC;
    memcpy(&io_buffer[5], data, LC);
    o_len = 5 + LC;
  }
  if (LE >= 0)
  {
    io_buffer[o_len] = (unsigned char) LE;
    o_len++;
  }

  i_len = sizeof(io_buffer);
  if (rsplen != NULL)
    if (i_len > (*rsplen + 1))
      i_len = (*rsplen + 1);

  rc = JaycosTransmitEx(io_buffer, o_len, io_buffer, &i_len);
  if (rc)
    return rc;

  if (i_len < 2)
    return TCL_OTHER_ERR;

  sw[0] = io_buffer[i_len - 2];
  sw[1] = io_buffer[i_len - 1];

  if (sw[0] == 0x61)
  {
    /* Proceed with GetResponse */
    return JaycosTransmit(0x00, 0xC0, 0x00, 0x00, NULL, 0, sw[1], rsp, rsplen,
                          status);
  }

  if (status != NULL)
  {
    *status = sw[0] << 8;
    *status |= sw[1];
  }

  if (rsp != NULL)
  {
    memcpy(rsp, io_buffer, i_len);
    if (rsplen != NULL)
      *rsplen = i_len;
  }

  return MI_OK;
}

/*
 * JayCOS Create File (user guide AE, chapter 3.1.8)
 * -------------------------------------------------
 */
short JaycosCreateFile(unsigned short FileID, unsigned char Descriptor,
                       unsigned char Extension,
                       unsigned char AccessConditions[3],
                       unsigned char AccessKeys[3],
                       unsigned char SizeH_RecCount,
                       unsigned char SizeL_RecSize)
{
  unsigned char cmddata[10];
  short rc;
  unsigned short status;

  cmddata[0] = Descriptor;
  cmddata[1] = Extension;
  if (AccessConditions != NULL)
  {
    cmddata[2] = AccessConditions[0];
    cmddata[3] = AccessConditions[1];
    cmddata[4] = AccessConditions[2];
  } else
  {
    /* Always */
    cmddata[2] = 0xFF;
    cmddata[3] = 0xFF;
    cmddata[4] = 0xFF;
  }
  if (AccessKeys != NULL)
  {
    cmddata[5] = AccessKeys[0];
    cmddata[6] = AccessKeys[1];
    cmddata[7] = AccessKeys[2];
  } else
  {
    /* Always */
    cmddata[5] = 0xFF;
    cmddata[6] = 0xFF;
    cmddata[7] = 0xFF;
  }
  cmddata[8] = SizeH_RecCount;
  cmddata[8] = SizeL_RecSize;

  rc =
    JaycosTransmit(0x80, 0xE0, (unsigned char) (FileID >> 8),
                   (unsigned char) FileID, cmddata, sizeof(cmddata), -1, NULL,
                   NULL, &status);
  if (rc)
    return rc;
  if (status != 0x9000)
    return status;              /* Wrong status word */

  return MI_OK;
}

/*
 * JayCOS Create Master File (user guide AE, chapter 3.1.7)
 * --------------------------------------------------------
 */
short JaycosCreateMF(void)
{
  unsigned short status;
  short rc;

  rc =
    JaycosTransmit(0x80, 0xE0, 0x3F, 0x00, NULL, 0, -1, NULL, NULL, &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * JayCOS Create DF (user guide AE, chapter 3.1.8)
 * -----------------------------------------------
 */
short JaycosCreateDF(unsigned short DirectoryIndex)
{
  unsigned char buf[10];
  unsigned short status;
  short rc;

  buf[0] = 0x00;                /* File Type = DF */
  buf[1] = 0x00;                /* (File Descriptor) */
  buf[2] = 0xFF;                /* Access conditions : ALWAYS */
  buf[3] = 0xFF;                /* Access conditions : ALWAYS */
  buf[4] = 0xFF;                /* Access conditions : ALWAYS */
  buf[5] = 0x00;                /* No key */
  buf[6] = 0x00;                /* No key */
  buf[7] = 0x00;                /* No key */
  buf[8] = 0x00;                /* (File Size H) */
  buf[9] = 0x00;                /* (File Size L) */

  rc =
    JaycosTransmit(0x80, 0xE0, (unsigned char) (DirectoryIndex / 0x0100),
                   (unsigned char) (DirectoryIndex % 0x0100), buf, 10, -1,
                   NULL, NULL, &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * JayCOS Create Transparent EF (user guide AE, chapter 3.1.8)
 * -----------------------------------------------------------
 */
short JaycosCreateTransparentEF(unsigned char FileIndex, unsigned char Size)
{
  unsigned char buf[10];
  unsigned short status;
  short rc;

  buf[0] = 0x20;                /* File Type = Transparent EF */
  buf[1] = 0x00;                /* (File Descriptor) */
  buf[2] = 0xFF;                /* Access conditions : ALWAYS */
  buf[3] = 0xFF;                /* Access conditions : ALWAYS */
  buf[4] = 0xFF;                /* Access conditions : ALWAYS */
  buf[5] = 0x00;                /* No key */
  buf[6] = 0x00;                /* No key */
  buf[7] = 0x00;                /* No key */
  buf[8] = 0x00;                /* (File Size H) */
  buf[9] = Size;                /* (File Size L) */

  rc =
    JaycosTransmit(0x80, 0xE0, 0x00, FileIndex, buf, 10, -1, NULL, NULL,
                   &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * JayCOS Select MF (user guide AE, chapter 3.1.10)
 * ------------------------------------------------
 */
short JaycosSelectMF(void)
{
  unsigned char buf[2];
  unsigned short status;
  short rc;

  buf[0] = 0x3F;
  buf[1] = 0x00;

  rc =
    JaycosTransmit(0x00, 0xA4, 0x08, 0x00, buf, 2, -1, NULL, NULL, &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * JayCOS Select DF (user guide AE, chapter 3.1.10)
 * ------------------------------------------------
 */
short JaycosSelectDF(unsigned short DirectoryIndex)
{
  unsigned char buf[2];
  unsigned short status;
  short rc;

  buf[0] = (unsigned char) (DirectoryIndex / 0x0100);
  buf[1] = (unsigned char) (DirectoryIndex % 0x0100);

  rc =
    JaycosTransmit(0x00, 0xA4, 0x08, 0x00, buf, 2, -1, NULL, NULL, &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * JayCOS Select EF (user guide AE, chapter 3.1.10)
 * ------------------------------------------------
 */
short JaycosSelectEF(unsigned short FileIndex)
{
  unsigned char buf[2];
  unsigned short status;
  short rc;

  buf[0] = (unsigned char) (FileIndex / 0x0100);
  buf[1] = (unsigned char) (FileIndex % 0x0100);

  rc =
    JaycosTransmit(0x00, 0xA4, 0x09, 0x00, buf, 2, -1, NULL, NULL, &status);

  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

short JaycosReadBinary(unsigned char Offset, unsigned char Length,
                       unsigned char Buffer[])
{
  unsigned char Temp[256 + 2];
  unsigned short ActualLength = Length + 2;
  short rc;

  if (ActualLength > sizeof(Temp))
    return TCL_OTHER_ERR;

  rc =
    JaycosTransmit(0x00, 0xB0, 0x00, Offset, NULL, 0, Length, Temp,
                   &ActualLength, NULL);
  if (rc != MI_OK)
    return rc;

  memcpy(Buffer, Temp, ActualLength - 2);

  return MI_OK;
}

short JaycosUpdateBinary(unsigned char Offset, unsigned char Length,
                         unsigned char Buffer[])
{
  return JaycosTransmit(0x00, 0xD6, 0x00, Offset, Buffer, Length, -1, NULL,
                        NULL, NULL);
}

/*
 * Jaycos Get ATR + Get Chip number (functional specification 3.1, chapter 4.7.1)
 * ----------------------------------------------------------------------------
 */
short JaycosGetAtr(unsigned char Atr[32], unsigned short *AtrLen)
{
  /* Select Desfire ISO applet */
  unsigned short status;
  int rsplen;
  short rc;

  rc =
    JaycosTransmit(0x80, 0xEC, 0x00, 0x00, NULL, 0, 0x0D, Atr, AtrLen,
                   &status);
  if (rc)
    return rc;
  if (status != 0x9000)
    return status;

  return MI_OK;
}

/*
 * Blank the Jaycos card (undocumented)
 * ------------------------------------
 */
static short JaycosWriteEeprom(unsigned short Offset, unsigned char Size,
                               unsigned char *Data)
{
  unsigned short status;
  short rc;
  unsigned char data[256] = { 0 };

  if (Data == NULL)
    rc =
      JaycosTransmit(0xB0, 0xB3, (unsigned char) (Offset / 0x0100),
                     (unsigned char) (Offset % 0x0100), data, Size, -1, NULL,
                     NULL, &status);
  else
    rc =
      JaycosTransmit(0xB0, 0xB3, (unsigned char) (Offset / 0x0100),
                     (unsigned char) (Offset % 0x0100), Data, Size, -1, NULL,
                     NULL, &status);
  if (rc)
    return rc;
  if (status != 0x9000)
    return status;              /* Wrong status word */

  return MI_OK;
}

short JaycosBlankCard(void)
{
  unsigned short offset;
  short rc;
  unsigned char data1[0x12] =
    { 0x2A, 0xA1, 0xFA, 0x1F, 0x8D, 0x09, 0x17, 0xF4, 0xD2, 0x2F, 0xC8, 0x71,
    0xD2, 0x6A, 0x25, 0xD2, 0xD0, 0xFF
  };
  unsigned char data2[0x01] = { 0x03 };

  rc = JaycosWriteEeprom(0x87FB, 0x04, NULL);
  if (rc)
    return rc;
  rc = JaycosWriteEeprom(0x87F0, 0x10, NULL);
  if (rc)
    return rc;
  rc = JaycosWriteEeprom(0x8000, 0x04, NULL);
  if (rc)
    return rc;
  for (offset = 0x8160; offset <= 0x8240; offset += 0x20)
  {
    rc = JaycosWriteEeprom(offset, 0x20, NULL);
    if (rc)
      return rc;
  }
  rc = JaycosWriteEeprom(0x8006, 0x02, NULL);
  if (rc)
    return rc;
  rc = JaycosWriteEeprom(0x8008, 0x10, NULL);
  if (rc)
    return rc;
  rc = JaycosWriteEeprom(0x87DE, 0x12, data1);
  if (rc)
    return rc;
  rc = JaycosWriteEeprom(0x87DD, 0x01, data2);
  if (rc)
    return rc;

  return MI_OK;
}
