#include <vcl.h>
#include <inifiles.hpp>
#pragma hdrstop

#include "uMain.h"
#include "uProgress.h"
#include "uWorker.h"

#include <stdio.h>

#pragma package(smart_init)
#pragma resource "*.dfm"

TfMain *fMain;
TWorker *Worker;

AnsiString toHEX(unsigned char *data, size_t size)
{
  char b[3];
  size_t i;
  AnsiString r = "";

  for (i = 0; i < size; i++)
  {
    sprintf(b, "%02X", data[i]);
    r += b;
  }
  return r;
}

__fastcall TfMain::TfMain(TComponent * Owner):TForm(Owner)
{
}

void __fastcall TfMain::FormShow(TObject * Sender)
{
  Enabled = false;
  fProgress->Show();
  Worker = new TWorker(false);
  Worker->Task(WORK_OPEN_READER);
}

void __fastcall TfMain::DataToDisplay()
{
  if (card_data.has_user)
  {
    eUserID->Text = toHEX(card_data.user_id, 8);
    eLastName->Text = card_data.last_name;
    eFirstName->Text = card_data.first_name;
    eBirthDate->Text = card_data.birth_date;
  }

  if (card_data.picture_size)
  {
    /* Try to display the picture */
    try
    {
      imgPicture->Picture->LoadFromFile(card_data.picture_file_name);
    }
    catch(...)
    {
      status->SimpleText = "Failed to display the picture";
      card_data.picture_size = false;
    }
    if (!card_data.picture_size)
    {
      ErasePicture();
    }
  } else
    ErasePicture();

  if (card_data.finger_size[0] || card_data.finger_size[1])
  {
    if (card_data.finger_size[0] && card_data.finger_size[1])
      lbFingerprint->Caption = "2 fingerprints loaded";
    else
      lbFingerprint->Caption = "1 fingerprint loaded";
    btnFingerVerify->Enabled = Worker->bReaderHasFingerprint;
  } else
  {
    lbFingerprint->Caption = "No fingerprint in the card";
    btnFingerVerify->Enabled = false;
  }
}

/*
 * Main -> Worker -> Main principle
 * --------------------------------
 *
 * Dialog with the reader is not implemented in the main thread of the application,
 * but in a "Worker" thread instead.
 * This give better user experience (no GUI latency during dialog with the reader).
 *
 * In our architecture, Main (VCL thread in Borland model) wakes Worker up with an
 * Event (the Event mechanism is implemented right in uWorker.cpp).
 * The reciprocity is not easy. Main can't be waiting on an Event. The only possibility
 * is to expose "callbacks" functions that will be called from the Worker thread in a
 * "synchronized" environment. Please refer to Borland's VCL and thread documentation if
 * this is unclear (yet this is a typical Borlandish, but we retrieve the same stuff in
 * Microsoft .NET programming model. Guess who have copied ?).
 * Anyway, the result is that we invoke worker thread with Worker->Task, and it call us
 * back with OnWorkerDone
 */
void __fastcall TfMain::OnWorkerDone(int WorkerAction, bool WorkerResult)
{
  if (WorkerAction == WORK_FIND_CARD)
  {
    if (WorkerResult)
    {
      status->SimpleText = "Card activated";

      if (card_data.is_type_a)
      {
        gbTypeB->Visible = false;
        eUid->Text = toHEX(card_data.iso.a.snr, card_data.iso.a.snrlen);
        if (card_data.is_tcl)
        {
          eAtsA->Text = toHEX(card_data.tcl.ats, card_data.tcl.atslen);
        } else
        {
          eAtsA->Text = "";
        }
        lbTypeA->Caption = "";
        gbTypeA->Visible = true;
      } else if (card_data.is_type_b)
      {
        gbTypeA->Visible = false;
        ePupi->Text = toHEX(card_data.iso.b.atq, 4);
        if (card_data.is_tcl)
        {
          eAtsB->Text = toHEX(card_data.tcl.ats, card_data.tcl.atslen);
        } else
        {
          eAtsB->Text = "";
        }
        lbTypeB->Caption = "";
        gbTypeB->Visible = true;
      }

      switch (card_data.is_type)
      {
        case CARD_DESFIRE:
          lbTypeA->Caption = "Desfire card";
          break;
        case CARD_MIFARE_1K:
          lbTypeA->Caption = "Mifare 1k card";
          break;
        case CARD_MIFARE_4K:
          lbTypeA->Caption = "Mifare 4k card";
          break;
        case CARD_MIFARE_UL:
          lbTypeA->Caption = "Mifare UltraLite card";
          break;
        case CARD_JAYCOS:
          lbTypeB->Caption = "Inseal Jaycos card";
          break;
      }

      /* We have a card ! */
      if (card_data.is_type != CARD_UNKNOWN)
      {
        if (after_find_do_task == WORK_WRITE_CARD)
        {
          /* We are to write the card, ask confirmation now ! */
          if (MessageBox
              (0,
               "Are you sure you want to write current data onto the selected card ?",
               "Confirmation",
               MB_YESNO | MB_ICONQUESTION | MB_TASKMODAL | MB_SETFOREGROUND)
              != IDYES)
            after_find_do_task = WORK_NONE;

        } else if (after_find_do_task == WORK_ERASE_CARD)
        {
          /* We are to erase the card, ask confirmation now ! */
          if (MessageBox
              (0,
               "Are you sure you want to (totally) erase the selected card ?",
               "Confirmation",
               MB_YESNO | MB_DEFBUTTON2 | MB_ICONSTOP | MB_TASKMODAL |
               MB_SETFOREGROUND) != IDYES)
            after_find_do_task = WORK_NONE;
        }

        /* Do next job if some */
        if (after_find_do_task > WORK_NONE)
        {
          Worker->Task((work_task_e) after_find_do_task);
          after_find_do_task = WORK_NONE;
        }
      }
    } else
    {
      card_data.is_type = CARD_ABSENT;
      status->SimpleText =
        "No card on the reader (or card no more available)";
    }
  } else if (WorkerAction == WORK_READ_CARD)
  {
    if (WorkerResult)
    {
      /* A card has been successfully read, let's display its data */
      DataToDisplay();
    }

  } else if (WorkerAction == WORK_WRITE_CARD)
  {


  } else if (WorkerAction == WORK_ENROLL)
  {
    /* Result of fingerprint enrollement */
    if (WorkerResult)
    {
      /* Successfull capture of fingerprint */
      btnFingerVerify->Enabled = Worker->bReaderHasFingerprint;
      btnFingerRemove->Enabled = true;
      btnFingerEnroll->Enabled = false;
      lbFingerprint->Caption = "Fingerprint ready";
    }
  } else if (WorkerAction == WORK_VERIFY)
  {
    /* Result of fingerprint verification */
    if (WorkerResult)
      lbFingerOK->Visible = true;
    else
      lbFingerKO->Visible = true;
  }

  /* If there is no error, we may hide the output console */
  if (WorkerResult)
  {
    Enabled = true;
    fProgress->Hide();
  } else
  {
    Enabled = false;
    fProgress->Show();
    fProgress->Caption = "Failed !";
    fProgress->btnClose->Enabled = true;
    fProgress->btnClose->SetFocus();
  }
}

void __fastcall TfMain::EraseFields()
{
  eUserID->Text = "";
  eFirstName->Text = "";
  eLastName->Text = "";
  eBirthDate->Text = "";
  ePurse->Text = "0";
  lbFingerprint->Caption = "No fingerprint";
  lbFingerOK->Visible = false;
  lbFingerKO->Visible = false;
  ErasePicture();
}

void __fastcall TfMain::ErasePicture()
{
  try
  {
    imgPicture->Picture->LoadFromFile("nobody.jpg");
  }
  catch(...)
  {

  }
}

void __fastcall TfMain::miNewClick(TObject * Sender)
{
  EraseFields();
  SetEditable(true);
}

void __fastcall TfMain::SetEditable(bool editable)
{
  pIdentity->Enabled = editable;
  gbPurse->Enabled = editable;
  if (editable)
  {
    eUserID->Color = clWindow;
    eFirstName->Color = clWindow;
    eLastName->Color = clWindow;
    eBirthDate->Color = clWindow;
    ePurse->Color = clWindow;
  } else
  {
    eUserID->ParentColor = true;
    eFirstName->ParentColor = true;
    eLastName->ParentColor = true;
    eBirthDate->ParentColor = true;
    ePurse->ParentColor = true;
  }
  btnPictureRemove->Enabled = (editable && card_data.picture_size);
  btnPictureBrowse->Enabled = (editable && !card_data.picture_size);
  btnFingerRemove->Enabled = (editable
                              && (card_data.finger_size[0]
                                  || card_data.finger_size[1])
                              && Worker->bReaderHasFingerprint);
  btnFingerEnroll->Enabled = (editable
                              && !(card_data.finger_size[0]
                                   || card_data.finger_size[1])
                              && Worker->bReaderHasFingerprint);
  btnFingerVerify->Enabled = (card_data.finger_size[0]
                              || card_data.finger_size[1])
    && Worker->bReaderHasFingerprint;
}


void __fastcall TfMain::miEditClick(TObject * Sender)
{
  SetEditable(true);
}

void __fastcall TfMain::miWriteClick(TObject * Sender)
{
  SetEditable(false);

  AnsiString s = eUserID->Text;

  while (s.Length() < 16)
    s = "0" + s;
  if (s.Length() > 16)
    s = s.SubString(1, 16);
  for (int i = 0; i < 8; i++)
  {
    card_data.user_id[i] = StrToIntDef("0x" + s.SubString(2 * i + 1, 2), 0);
  }

  card_data.last_name = eLastName->Text;
  card_data.first_name = eFirstName->Text;
  card_data.birth_date = eBirthDate->Text;

  after_find_do_task = WORK_WRITE_CARD;

  Enabled = false;
  Worker->Task(WORK_FIND_CARD);
}

void __fastcall TfMain::miEraseClick(TObject * Sender)
{
  after_find_do_task = WORK_ERASE_CARD;
  Enabled = false;
  Worker->Task(WORK_FIND_CARD);
}

void __fastcall TfMain::miReadClick(TObject * Sender)
{
  SetEditable(false);
  after_find_do_task = WORK_READ_CARD;
  Enabled = false;
  Worker->Task(WORK_FIND_CARD);
}


void __fastcall TfMain::FormCreate(TObject * Sender)
{
  char t[1024];

  GetTempPath(1024, t);
  temp_picture_file_name = t;
  temp_picture_file_name += "\\mycard.jpg";

  TIniFile *ini;

  ini = new TIniFile(ExtractFileDir(Application->ExeName) + "\\csbinfo.ini");
  AnsiString str_company = ini->ReadString("company", "name", "SprinCard");
  AnsiString str_logo =
    ini->ReadString("company", "logo", "logo_pro-active.bmp");
  AnsiString str_product =
    ini->ReadString("product", "name", "CSB / SpringProx");
  delete ini;

  TPicture *pic;

  pic = new TPicture();
  try
  {
    pic->LoadFromFile(str_logo);
    imgLogo->Left = pTop->Width - 8 - pic->Width;
    imgLogo->Picture->Assign(pic);
  } catch(...)
  {
    /* No picture ! */
  }
  delete pic;

  Caption = str_company + " - " + Caption;
  Application->Title = Caption;
}

//---------------------------------------------------------------------------

void __fastcall TfMain::btnPictureBrowseClick(TObject * Sender)
{
  if (dlgOpenPicture->Execute())
  {
    TPicture *pic;

    pic = new TPicture();
    try
    {
      pic->LoadFromFile(dlgOpenPicture->FileName);
      imgPicture->Picture->Assign(pic);
      card_data.picture_file_name = dlgOpenPicture->FileName;
      card_data.picture_size = 1; /* Actual size will be checked when writing onto the card */
    }
    catch(...)
    {
      card_data.picture_size = 0;
      ShowMessage("Picture file is either invalid or unavailable");
      ErasePicture();
    }
    delete pic;
  }
}

void __fastcall TfMain::btnFingerEnrollClick(TObject * Sender)
{
  lbFingerOK->Visible = false;
  lbFingerKO->Visible = false;
  Enabled = false;
  Worker->Task(WORK_ENROLL);
}


void __fastcall TfMain::btnFingerVerifyClick(TObject * Sender)
{
  lbFingerOK->Visible = false;
  lbFingerKO->Visible = false;
  Enabled = false;
  Worker->Task(WORK_VERIFY);
}


void __fastcall TfMain::btnFingerRemoveClick(TObject * Sender)
{
  lbFingerOK->Visible = false;
  lbFingerKO->Visible = false;
  lbFingerprint->Caption = "Fingerprints erased";
  card_data.finger_size[0] = false;
  card_data.finger_size[1] = false;
  btnFingerRemove->Enabled = false;
  btnFingerVerify->Enabled = false;
  btnFingerEnroll->Enabled = Worker->bReaderHasFingerprint;
}

void __fastcall TfMain::miQuitClick(TObject * Sender)
{
  Close();
}

void __fastcall TfMain::btnPictureRemoveClick(TObject * Sender)
{
  ErasePicture();
  card_data.picture_size = 0;
  btnPictureRemove->Enabled = false;
  btnPictureBrowse->Enabled = true;
}

//---------------------------------------------------------------------------

